import { fromEvent } from 'rxjs';

/**
 * Base class for commands sent to the MetaBox.
 */
class CommandBase {
}

/**
 * Represents a command to toggle the Unreal Overlay Interface Menu.
 *
 * @remarks
 * This action sends a message to the MetaBox API to toggle the visibility of the Unreal overlay UI.
 *
 * @example
 * import { ShowOverlayInterface,Communicator } from '@3dsource/metabox-front-api';
 *
 * //...Assume that the integration is already implemented
 * window.env3DSource.apiReady = (api:Communicator) => {
 *   api.setCommandToMetaBox(new ShowOverlayInterface(true));
 * };
 */
class ShowOverlayInterface extends CommandBase {
    /**
     * Creates an instance of ShowOverlayInterface.
     *
     * @param visible - A flag indicating whether the Unreal overlay UI should be visible.
     */
    constructor(visible) {
        super();
        this.data = {
            action: 'showOverlayInterface',
            payload: {
                visible,
            },
        };
    }
}

/**
 * Represents a command to toggle the Embedded MetaBox Menu.
 *
 * @example
 * import { ShowEmbeddedMenu,Communicator } from '@3dsource/metabox-front-api';
 *
 * //...Assume that the integration is already implemented
 * window.env3DSource.apiReady = (api:Communicator) => {
 *   api.setCommandToMetaBox(new ShowEmbeddedMenu(true));
 * };
 */
class ShowEmbeddedMenu extends CommandBase {
    /**
     * Creates an instance of ShowEmbeddedMenu.
     *
     * @param visible - A flag indicating whether the embedded menu should be visible.
     */
    constructor(visible) {
        super();
        this.data = {
            action: 'showEmbeddedMenu',
            payload: {
                visible,
            },
        };
    }
}

/**
 * Represents a command to set the environment by its ID.
 *
 * @example
 * import { SetEnvironment,Communicator } from '@3dsource/metabox-front-api';
 *
 * //...Assume that the integration is already implemented
 * window.env3DSource.apiReady = (api:Communicator) => {
 *   // Change the environment to '55555555-1234-1234-1234-01234567890'
 *   api.sendCommandToMetaBox(new SetEnvironment('55555555-1234-1234-1234-01234567890'));
 * };
 */
class SetEnvironment extends CommandBase {
    /**
     * Creates an instance of SetEnvironment.
     *
     * @param environmentId - The environment ID.
     */
    constructor(environmentId) {
        super();
        this.data = {
            payload: {
                id: environmentId,
            },
            action: 'loadLevel',
        };
    }
}

/**
 * Represents a command to set a product by its ID.
 *
 * @remarks
 * This action sends a message to the MetaBox API to set a product using the provided product ID.
 *
 * @example
 * import { SetProduct,Communicator } from '@3dsource/metabox-front-api';
 *
 * //...Assume that the integration is already implemented
 * window.env3DSource.apiReady = (api:Communicator) => {
 *   api.setCommandToMetaBox(new SetProduct(
 *     'ffea6b5c-3a8a-4f56-9417-e605acb5cca3'
 *   ));
 * };
 */
class SetProduct extends CommandBase {
    /**
     * Creates an instance of SetProduct.
     *
     * @param productId - The product ID.
     */
    constructor(productId) {
        super();
        this.data = {
            payload: {
                productId,
            },
            action: 'setProduct',
        };
    }
}

/**
 * Represents a command to get a PDF from Metabox.
 * @remarks
 * This class sends a message to the MetaBox API to generate a PDF based on the current configuration.
 * This action does not require any parameters.
 *
 * **Important**: Invoking this command will buffer it until the viewport is ready. Only then will the request be processed.
 *
 * @example
 * import { GetPdf,Communicator } from '@3dsource/metabox-front-api';
 * window.env3DSource.apiReady = (api:Communicator) => {
 *     api.sendCommandToMetaBox(new GetPdf());
 * };
 */
class GetPdf extends CommandBase {
    /**
     * Creates an instance of GetPdf.
     *
     * @remarks
     * This constructor does not require any parameters.
     */
    constructor() {
        super();
        this.data = {
            action: 'getPdf',
        };
    }
}

/**
 * Represents a command to set a material by its ID.
 *
 * @example
 * import { SetMaterial,Communicator } from '@3dsource/metabox-front-api';
 *
 * //...Assume that the integration is already implemented
 * window.env3DSource.apiReady = (api:Communicator) => {
 *   api.sendCommandToMetaBox(new SetMaterial(
 *     'carpaint',
 *     'dd829d6e-9200-47a7-8d5b-af5df89b7e91',
 *   ));
 * };
 */
class SetMaterial extends CommandBase {
    /**
     * Creates an instance of SetMaterial.
     *
     * @param slotId - The slot ID.
     * @param materialId - The material ID.
     */
    constructor(slotId, materialId) {
        super();
        this.data = {
            payload: {
                slotId,
                materialId,
            },
            action: 'setMaterialById',
        };
    }
}

/**
 * Represents a command to set an environment material by its ID.
 *
 * @example
 * import { SetEnvironmentMaterial,Communicator } from '@3dsource/metabox-front-api';
 *
 * //...Assume that the integration is already implemented
 * window.env3DSource.apiReady = (api:Communicator) => {
 *    api.sendCommandToMetaBox(new SetEnvironmentMaterial(
 *     'carpaint',
 *     'dd829d6e-9200-47a7-8d5b-af5df89b7e91',
 *    ));
 * };
 */
class SetEnvironmentMaterial extends CommandBase {
    /**
     * Creates an instance of SetEnvironmentMaterial.
     *
     * @param slotId - The slot ID.
     * @param materialId - The material ID.
     */
    constructor(slotId, materialId) {
        super();
        this.data = {
            payload: {
                slotId,
                materialId,
            },
            action: 'setEnvironmentMaterialById',
        };
    }
}

/**
 * Represents a command to get fast render.
 *
 * **Important**: Invoking this command will buffer it until the viewport is ready. Only then will the request be processed.
 *
 * @example
 * import { GetScreenshot,Communicator,saveImage } from '@3dsource/metabox-front-api';
 * window.env3DSource.apiReady = (api:Communicator) => {
 *   // Listen for events from the MetaBox API
 *   api.addEventListener('screenshot', (data) => {
 *     // Process the fast render response
 *    saveImage(data, 'Render.png');
 *   });
 *   api.sendCommandToMetaBox(new GetScreenshot('image/png', { x: 1024, y: 1024 }));
 * };
 */
class GetScreenshot extends CommandBase {
    /**
     * Constructs an instance of GetScreenshot.
     * @param mimeType - The output format.
     * @param size - Optional size in pixels.
     */
    constructor(mimeType, size) {
        super();
        this.data = {
            payload: {
                format: mimeType,
                size,
            },
            action: 'getScreenshot',
        };
    }
}

/**
 * @internal
 * @hidden
 * Represents a command to send Unreal command.
 */
class UnrealCommand extends CommandBase {
    constructor(payload) {
        super();
        this.data = {
            payload,
            action: 'sendCommandToUnreal',
        };
    }
}
/**
 * @deprecated, use `UnrealCommand` instead.
 */
const SendCommandToUnreal = UnrealCommand;

/**
 * @experimental
 * @internal
 * @hidden
 * @remarks
 * Represents a command to watch Unreal callbacks from Unreal engine.
 * Can be useful for custom solutions.
 */
class WatchCallbacks extends CommandBase {
    /**
     * Creates an instance of WatchCallbacks.
     *
     * @remarks
     * This constructor does not require any parameters.
     */
    constructor() {
        super();
        this.data = {
            action: 'watchCallbacks',
        };
    }
}

/**
 * EventDispatcher is a class that manages event listeners and dispatches events to them.
 */
class EventDispatcher {
    constructor() {
        /**
         * Storage for callback listeners by message type.
         */
        this.listeners = [];
    }
    destroy() {
        this.listeners = [];
    }
    /**
     * Adds an event listener for receiving specific types of messages.
     *
     * @param messageType - The message type to listen for.
     * @param callback - The callback function to execute when a message is received.
     */
    addEventListener(messageType, callback) {
        this.listeners.push({ messageType, callback });
        return this;
    }
    /**
     * Dispatches an event to all listeners of a specific message type.
     *
     * @param messageType - The message type.
     * @param data - The data associated with the event.
     */
    dispatchEvent(messageType, data) {
        this.listeners
            .filter((listener) => listener.messageType === messageType)
            .forEach((listener) => listener.callback(data));
        return this;
    }
    /**
     * Removes an event listener for a specific type of message.
     *
     * @param messageType - The message type.
     * @param callback - The callback function to remove.
     */
    removeEventListener(messageType, callback) {
        this.listeners = this.listeners.filter((listener) => !(listener.messageType === messageType && listener.callback === callback));
        return this;
    }
}

/**
 * Handles messaging between the host page and embedded MetaBox content.
 * @internal Use {@link Communicator.createInstance} or the {@link integrateMetaBox} helper to instantiate.
 */
class Communicator extends EventDispatcher {
    /**
     * Constructs a Communicator, replacing any existing instance, and begins listening for messages.
     * @internal
     */
    constructor() {
        var _a;
        super();
        /**
         * Bound handler for incoming postMessage events.
         */
        this.binder = this.handleMessageReceived.bind(this, 'metaboxData');
        (_a = Communicator.instance) === null || _a === void 0 ? void 0 : _a.destroy();
        Communicator.instance = this;
        window.addEventListener('message', this.binder);
    }
    /**
     * Cleans up resources and stops listening for messages.
     */
    destroy() {
        super.destroy();
        window.removeEventListener('message', this.binder);
    }
    /**
     * Posts a command to the MetaBox iframe.
     * @param command - An action command containing data to send.
     */
    sendCommandToMetaBox(command) {
        const { data } = command;
        const iframe = document.getElementById('embeddedContent');
        if (!(iframe === null || iframe === void 0 ? void 0 : iframe.contentWindow)) {
            console.warn('MetaBox IFrame not found or not ready.');
            return;
        }
        const message = {
            host: 'metaBoxHost',
            payload: Object.assign({}, data),
        };
        // Replace '*' with a specific origin in production for better security.
        iframe.contentWindow.postMessage(message, '*');
    }
    /**
     * Filters and dispatches incoming messages from MetaBox.
     * @param messageType - Expected message type for filtering ('metaboxData').
     * @param event - The postMessage event received on window.
     */
    handleMessageReceived(messageType, event) {
        var _a;
        // Optionally validate event.origin for security.
        if (messageType !== 'metaboxData' || ((_a = event.data) === null || _a === void 0 ? void 0 : _a.host) !== 'metabox') {
            return;
        }
        const { eventType, payload } = event.data.payload;
        this.dispatchEvent(eventType, payload);
    }
    /**
     * Listens for MetaBox to signal readiness, then initializes communicator.
     * @param apiReadyCallback - Called with the new Communicator once MetaBox is loaded.
     */
    static createInstance(apiReadyCallback) {
        const startHandler = (event) => {
            var _a;
            const message = event.data;
            if (((_a = message === null || message === void 0 ? void 0 : message.payload) === null || _a === void 0 ? void 0 : _a.action) === 'appLoaded') {
                apiReadyCallback(new Communicator());
                window.removeEventListener('message', startHandler);
            }
        };
        window.addEventListener('message', startHandler);
    }
    /**
     * Registers an event listener for messages dispatched by MetaBox.
     * @override
     */
    addEventListener(messageType, callback) {
        return super.addEventListener(messageType, callback);
    }
    /**
     * Dispatches a typed event to all registered listeners.
     * @override
     */
    dispatchEvent(messageType, data) {
        return super.dispatchEvent(messageType, data);
    }
    /**
     * Removes a previously registered event listener.
     * @override
     */
    removeEventListener(messageType, callback) {
        return super.removeEventListener(messageType, callback);
    }
}
/**
 * Singleton reference to the current communicator instance.
 */
Communicator.instance = null;

/**
 * Integrates the MetaBox into the provided layout.
 *
 * @remarks
 * This function injects an iframe into a specified container element in the DOM and initializes
 * the communicator for interacting with the MetaBox embedded content.
 *
 * @param iframeSrc - The source URL of the MetaBox iframe.
 * Additional url parameters:
 * ```javascript
 * sidebar=false - disables the internal MetaBox menu
 * introImage=https://example.com/image.png - sets the intro image
 * ```
 * i.e. final link will look like https://metabox.3dsource.com/metabox-configurator/basic/your-configurator-id?sidebar=false&introImage=https://example.com/image.png
 *
 * @param containerId - The ID of the layout element. Defaults to 'embed3DSource'.
 * @param apiReadyCallback
 * @throws Error if the container element with the specified ID is not found.
 *
 * @example
 * import {
 *  integrateMetaBox,
 * } from '@3dsource/metabox-front-api';
 *
 * integrateMetaBox(
 *   'https://metabox.3dsource.com/metabox-configurator/basic/12345678-1234-1234-1234-01234567890?introImage=https://example.com/intro.webp&sidebar=false',
 *   'embed3DSource'
 * );
 */
function integrateMetaBox(iframeSrc, containerId = 'embed3DSource', apiReadyCallback) {
    const container = document.getElementById(containerId);
    if (!container) {
        throw new Error(`Container element with id ${containerId} not found`);
    }
    Communicator.createInstance(apiReadyCallback);
    const style = document.createElement('style');
    style.innerHTML = `
        #${containerId} {
          width: 100%;
          height: 100%;
          overflow: hidden;
          position: relative;
        }
    `;
    document.head.appendChild(style);
    const iframe = document.createElement('iframe');
    iframe.setAttribute('referrerPolicy', 'no-referrer-when-downgrade');
    iframe.setAttribute('id', 'embeddedContent');
    iframe.style.border = '0';
    iframe.style.width = '100%';
    iframe.style.height = '100%';
    iframe.style.overflow = 'hidden';
    iframe.src = iframeSrc;
    container.appendChild(iframe);
}

/**
 * Wrapper helper function to use it with rxJs same as `fromEvent` to listen to events from the Communicator.
 * @param target
 * @param eventName
 */
function fromCommunicatorEvent(target, eventName) {
    return fromEvent(target, eventName);
}

/**
 * Saves an image by triggering a download.
 *
 * @remarks
 * This function creates an anchor element, sets its `href` attribute to the provided image URL,
 * and triggers a click event to initiate a download with the specified filename.
 *
 * @param imageUrl - The URL of the image to save.
 * @param filename - The name of the file to save.
 */
function saveImage(imageUrl, filename) {
    const a = document.createElement('a');
    a.href = imageUrl;
    a.download = filename;
    document.body.appendChild(a);
    a.click();
    document.body.removeChild(a);
}

/**
 * Pauses execution for a specified duration.
 *
 * @remarks
 * This function returns a promise that resolves after the given number of milliseconds.
 *
 * @param time - The time to sleep in milliseconds.
 * @returns A promise that resolves after the specified duration.
 */
function sleep(time) {
    return new Promise((resolve) => setTimeout(resolve, time));
}

/**
 * Sets the URL parameters based on the provided state.
 *
 * @remarks
 * This function updates the current URL by replacing its search parameters with the key-value pairs from the provided state object.
 *
 * @param state - An object containing key-value pairs to set as URL parameters.
 */
function setUrlParams(state) {
    const url = new URL(window.location.href);
    url.search = '';
    Object.entries(state).forEach(([key, value]) => {
        url.searchParams.set(key, value);
    });
    history.replaceState(null, '', url.toString());
}
/**
 * Retrieves the URL parameters as an object.
 *
 * @remarks
 * This function parses the current URL's search parameters and returns them as a key-value object.
 *
 * @returns An object containing the URL parameters.
 */
function getUrlParams() {
    const urlParams = new URLSearchParams(window.location.search);
    const selections = {};
    urlParams.forEach((value, key) => {
        selections[key] = value;
    });
    return selections;
}

export { CommandBase, Communicator, GetPdf, GetScreenshot, SendCommandToUnreal, SetEnvironment, SetEnvironmentMaterial, SetMaterial, SetProduct, ShowEmbeddedMenu, ShowOverlayInterface, UnrealCommand, WatchCallbacks, fromCommunicatorEvent, getUrlParams, integrateMetaBox, saveImage, setUrlParams, sleep };
//# sourceMappingURL=index.esm.js.map
