/**
 * Represents a configurator object with its configuration, selected materials,
 * product identifier, and associated environment.
 */
export interface ConfiguratorEnvelope {
    /**
     * A reference to the universal configurator.
     */
    configurator: UniversalConfigurator;
    /**
     * The unique product identifier.
     */
    productId: string;
    /**
     * A record mapping product material selections.
     */
    productMaterialsIds: SelectedIds;
    /**
     * @deprecated,
     * use `productMaterialsIds` instead.
     * @hidden
     */
    selectedMaterials: SelectedIds;
    /**
     * The unique environment identifier.
     */
    environmentId: string;
    /**
     * A record mapping environment material selections.
     */
    environmentMaterialsIds: SelectedIds;
    /**
     * A configurator quote for sales
     */
    configuration: SelectedConfigurationForQuote | null;
}
/**
 * Represents the universal configurator with its basic properties and components.
 */
export interface UniversalConfigurator {
    /** Unique identifier for the configurator. */
    id: string;
    /** Indicates if the configurator is active. */
    /** Label for the product collection. */
    productCollectionLabel: string;
    /** The name of the configurator. */
    name: string;
    /** The camera settings used in the configurator. */
    /** Client information associated with the configurator. */
    /** A list of products for the configurator. */
    products: UniversalConfiguratorProduct[];
    /** A list of available environments. */
    environments: UniversalConfiguratorEnvironment[];
    /** GraphQL typename for the configurator. */
    __typename: 'UniversalConfigurator';
}
/**
 * Represents a product within the universal configurator.
 */
export interface UniversalConfiguratorProduct {
    /** The product's unique identifier. */
    productId: string;
    /** Position index of the product. */
    position: number;
    /** Display label for the product. */
    label: string;
    /** Optional camera settings for the product. */
    /** The detailed product information. */
    product: Product;
    /** GraphQL typename for the product. */
    __typename: 'UniversalConfiguratorProduct';
}
/**
 * Represents product details including visual assets and configuration.
 */
export interface Product {
    /** The title of the product. */
    title: string;
    /** List of thumbnails for the product. */
    thumbnailList: Thumbnail[];
    /** Available slots for the product. */
    slots: Slot[];
    /** List of metaProperties for the product (optional). */
    metaProperties: MetaProperty[];
    /** The model information associated with the product. */
    /** Unique identifier for the product. */
    id: string;
    /** Unique external id for the product. */
    externalId: string;
    /** Showcase details for the product. */
    /** GraphQL typename for the product. */
    __typename: 'Product';
}
/**
 * Represents a thumbnail image entity with dimensions and URL path.
 */
export interface Thumbnail {
    /** Width of the thumbnail. */
    width: number;
    /** Height of the thumbnail. */
    height: number;
    /** URL path to the thumbnail image. */
    urlPath: string;
    /** GraphQL typename (as a string, flexible type). */
    __typename: 'Thumbnail';
}
/**
 * Represents a slot within a product that may allow material customization.
 */
export interface Slot {
    /** Unique identifier for the slot. */
    id: string;
    /** Display a label for the slot. */
    label: string;
    /** A list of enabled materials for the slot. */
    enabledMaterials: Material[];
    /** GraphQL typename for the slot. */
    __typename: 'Slot';
}
/**
 * Represents a metaProperty within a product that may allow business logic customization.
 */
export interface MetaProperty {
    /** Unique identifier for the metaProperty. */
    id: string;
    /** Display the name for the metaProperty. */
    name: string;
    /**  Value for the metaProperty. */
    value: string;
    /**  Type for the metaProperty. */
    type: string;
    /** GraphQL typename for the metaProperty. */
    __typename: 'MetaProperty';
}
/**
 * Represents a material that can be applied to a product slot.
 */
export interface Material {
    /** Unique identifier for the material. */
    id: string;
    /** Title or name of the material. */
    title: string;
    /** Unique external id for the material. */
    externalId: string;
    /** List of thumbnails for the material. */
    thumbnailList: Thumbnail[];
    /** The model information associated with the material. */
    /** GraphQL typename for the material. */
    __typename: 'Material';
}
/**
 * Represents the environment settings for a universal configurator.
 */
export interface UniversalConfiguratorEnvironment {
    /** Unique identifier for the environment. */
    environmentId: string;
    /** Position index of the environment. */
    position: number;
    /** Display label for the environment. */
    label: string;
    /** Detailed environment configuration. */
    environment: Environment;
    /** GraphQL typename for the environment. */
    __typename: 'UniversalConfiguratorEnvironment';
}
/**
 * Represents an environment with associated media and configuration.
 */
export interface Environment {
    /** Unique identifier for the environment. */
    id: string;
    /** Title or name of the environment. */
    title: string;
    /** Flag indicating if UDS is enabled. */
    /** The north yaw of the sun in the environment. */
    /** The UDS hour setting for the environment. */
    /** GraphQL typename for the environment. */
    __typename: 'Environment';
    /** List of thumbnails for the environment (optional). */
    thumbnailList: Thumbnail[];
    /** List of slots for the environment (optional). */
    slots: Slot[];
}
/**
 * Defines the structure for a mapping of selected materials.
 */
export type SelectedIds = Record<string, Record<string, string>>;
/**
 * Defines the structure for a mapping of the selected configuration.
 */
export interface SelectedConfigurationForQuote {
    product: {
        externalId: string;
        title: string;
        slots: {
            label: string;
            material: {
                externalId: string;
                title: string;
            };
        }[];
    };
    environment?: {
        title: string;
        slots: {
            label: string;
            material: {
                externalId: string;
                title: string;
            };
        }[];
    };
}
